# encoding: utf-8
require "html2erb/version"
require 'nkf'
require "nokogiri"

class Html2erb
  attr_accessor :replace_map, :file_suffix, :url_path_map

  def initialize(opt = {})
    opt.each do|k, v|
      self.send("#{k}=", v)
    end
  end

  def run(file_name)
    Dir.glob(file_name).each do |path|
      f = open(path, "r+") 
      html = f.read
      f.close

      erb_file_path = erb_file_path(path)
      f = open(erb_file_path, "w")
      f.write replace_tags(html)
      f.close

      replace_tags(erb_file_path)
    end
  end

  def erb_file_path(org_path)
    file_name = File.basename(org_path)
    org_file_name = file_name.sub('.html', "#{@file_suffix}.html.erb")
    org_path.sub(file_name, org_file_name)
  end

  def replace_tags(html)
    src_encoding = NKF.guess(html).to_s
    unless src_encoding == 'UTF-8'
      html.encode!('UTF-8', src_encoding, invalid: :replace, undef: :replace, replace: '?')
      html.gsub! "\r\n", "\n"
    end

    @replace_map.each do |regex, replacement|
      html.sub! regex, replacement 
    end

    a_tag html 
    img_tag html 

    html
  end

  def img_tag(html)
    html.gsub!(/<img .*?\/>/m) do |link|
      tag = Nokogiri::HTML.parse(link, nil, 'utf-8').xpath("//img")[0]
      variables = dom_variables('img', tag).join(", ")
      "<%= image_tag(#{variables}) %>"
    end
    html
  end

  def a_tag(html)
    html.gsub!(/<a .*?<\/a>/m) do |link|
      tag = Nokogiri::HTML.parse(link, nil, 'utf-8').xpath("//a")[0]
      href, opt = dom_variables('a', tag)

      # <a> タグの中身
      label = inner_html tag
      href  = '""' if href == ''
      case href
      when '"how-to.html"'  then href = 'howto_path'
      when '"psychic.html"' then href = 'fortuneteller_index_path'
      when '"about.html"'   then href = 'about_path'
      when '"index.html"'   then href = 'root_path'
      end
      variables = [label.join(" + "), href, opt]
      variables.delete ''
      "<%= link_to(#{variables.join(", ").strip}) %>"
    end
    html
  end

  def dom_variables(key, dom)
    case key 
    when 'img' then key = 'src'
    when 'a'   then key = 'href'
    end

    opt = {}
    target = ''
    dom.each do|k,v|
      if k == key
        # ここは、html の <img> のパスにあわせて変更しなければいけない
        if key == 'src'
          v = v.sub('/m/img/', 'mobile/')
            .sub(/\Aimg\//, 'mobile/')
            .sub("images/site/", "pc/")
        end

        target = '"' + v + '"'
      else
        opt[k.to_sym] = v if v != ''
      end
    end
    [target, opt.to_pretty_s]
  end

  def inner_html(dom)
    label = []
    dom.children.each do|child|
      case child.name 
      when 'img', 'span', 'text' then label << to_content_tag(child)
      when 'br'                  then label << 'raw("<br>")'
      else
        raise "undefined tag: #{child.name}"
      end
    end

    label.delete ""
    label.length == 0 ? ['""'] : label
  end

  def to_content_tag(dom)
    if dom.name == 'text'
      if(dom.to_s =~ /\A\n +\z/)
        return '' 
      else
        return '"' + dom.to_s + '"' 
      end
    end

    if(dom.name == 'img')
      src, opt = dom_variables('img', dom)
      return "image_tag(#{src}, #{opt})"
    end

    opt = {}
    value = ''
    dom.each do|k, v|
      opt[k.to_sym] = v
    end

    if dom.children 
      dom.children.each do |child|
        value << to_content_tag(child)
      end
    end

    variables = [":#{dom.name}", value, opt.to_pretty_s]
    "content_tag(#{variables.join(", ").strip})"
  end
end

class Hash
  # ハッシュを [ hoge: 'fuga' ] 形式で表現した文字列を返す
  def to_pretty_s
    buf = []
    self.each{|k,v|
      buf << "#{k}: '#{v}'" if v != ''
    }
    buf.join(", ")
    # self.to_s.gsub(':', '') # : を取り除く
    # .gsub('=>', ': ') # => を : に
    # .gsub(/\A\{/, '') # 先頭のブラケット
    # .gsub(/\}\z/, '') # 末尾のブラケット
    # .strip.gsub(/,\z/, '') # 末尾のカンマ
  end
end
